require 'uu/os/lang/uniform_hash'

module UU
  module OS
    module HTTP

      # Object representing HTTP headers with Hash-like API.
      class HTTPHeaders < UU::OS::Lang::UniformHash

        # Creates new instance of HTTPHeaders.
        # @private
        def initialize(obj = nil) # :nodoc:
          if obj.kind_of?Hash
            self.replace(obj)
          else
            super(obj)
          end
        end

        # Appends value to header of given name.
        # @param [Symbol, String] name Header name.
        # @param [String] value Header value.
        # @return [void]
        def add(name, value)
          if has_key?(name)
            self[name].concat(transform_value(value))
          else
            self[name] = value
          end
        end

        # Returns true if headers contains header with
        # given name and optional value.
        # @param [Symbol, String] name Header name.
        # @param [String] value Header value snippet (regular expression).
        # @return [TrueClass, FalseClass] True if header with given name and optional value snippet exists.
        def contains(name, value = nil)
          return false if !has_key?(name)
          return true if value.nil?
          self[name].each do |val|
            return true if val =~ /#{value}/
          end
          return false
        end

        # Translates all given keys to lowercase symbol.
        # Also replaces all underscores (which are not
        # allowed in HTTP headers) with dash.
        # @param [String, Symbol] key Key to be transformed
        # @return [Symbol] Transformed key
        # @private
        def transform_key(key) # :nodoc:
          return nil if key.nil?
          key.to_s.gsub('-','_').downcase.to_sym
        end

        # Transforms all given values to array of values.
        # @param [Object] value Value to be transformed
        # @return [Array<Object>] Transformed value
        # @private
        def transform_value(value) # :nodoc:
          if (value.kind_of?Array)
            return value
          else
            return [value]
          end
        end

      end

    end
  end
end
